// ****************************************************************************
// copyright (c) 2000-2002 Horst Knorr <hk_classes@knoda.org>
// This file is part of the hk_classes library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
//$Revision: 1.17 $
#include "hk_dsvisible.h"
#include "hk_database.h"
#include "hk_presentation.h"
#include "hk_form.h"
#include "hk_datasource.h"

hk_dsvisible::hk_dsvisible(hk_presentation* presentation):hk_visible(presentation)
{
#ifdef HK_DEBUG
    hkdebug("hk_dsvisible::constructor");
#endif
    p_datasource=NULL;

#ifdef HK_DEBUG
    hkdebug("hk_dsvisible::constructor",p_presentation==NULL?"presentation==NULL":"presentation!=NULL");
#endif
    p_presentationdatasource=-1;
    p_readonly=false;
    widget_specific_enable_disable();
    internal_dsrefresh=false;
    p_already_handled=false;

}


hk_dsvisible::~hk_dsvisible(void)
{
#ifdef HK_DEBUG
    hkdebug("hk_dsvisible::destructor");
#endif
    if (p_datasource!=NULL)p_datasource->visible_remove(this);
//    else  cout <<"datasource==NULL"<<endl;
}


void hk_dsvisible::set_datasource(hk_datasource* d)
{
#ifdef HK_DEBUG
    hkdebug("hk_dsvisible::set_datasource");
#endif
    if (p_presentation!=NULL)if (!internal_dsrefresh)return;
#ifdef HK_DEBUG
    hkdebug("hk_dsvisible::datasource Datenquelle anfgen");
#endif
    if (p_datasource!=NULL)
    {
        p_datasource->visible_remove(this);
        p_datasource=NULL;
    }
    if (d!=NULL)
    {
        p_datasource=d;
        p_datasource->visible_add(this);
        if (p_datasource->is_enabled())datasource_enable();
#ifdef HK_DEBUG
        hkdebug("datasource add");
#endif
    }
    widget_specific_enable_disable();
}


hk_datasource* hk_dsvisible::datasource(void)
{
    return p_datasource;
}


bool hk_dsvisible::set_presentationdatasource(long n,bool registerchange)
{
#ifdef HK_DEBUG
    hkdebug("hk_dsvisible::set_presentationdatasource");
#endif
    if (p_presentation==NULL)
    {
#ifdef HK_DEBUG
        hkdebug("hk_dsvisible::presentationdatasource presentation==NULL");
#endif

        return false;
    }
    widget_specific_presentationdatasource(n);
    internal_dsrefresh=true;
    p_presentationdatasource=n;
    hk_datasource* d=p_presentation->get_datasource(n);
    set_datasource(d);
    internal_dsrefresh=false;
    has_changed(registerchange);

    return true;

}


long    hk_dsvisible::presentationdatasource(void)
{

    return p_presentationdatasource;
}


void hk_dsvisible::row_change()
{
#ifdef HK_DEBUG
    hkdebug("dsvisible::row_change");
#endif
    widget_specific_row_change();

}


void    hk_dsvisible::after_store_changed_data(void)
{
#ifdef HK_DEBUG
    hkdebug("dsvisible::after_store_changed_data");
#endif
    widget_specific_row_change();
}


void hk_dsvisible::datasource_delete()
{
#ifdef HK_DEBUG
    hkdebug("dsvisible::datasource_delete");
#endif
    p_datasource=NULL;
}


bool hk_dsvisible::datasource_disable()
{
#ifdef HK_DEBUG
    hkdebug("dsvisible::datasource_disable");
#endif
    widget_specific_disable();
    widget_specific_enable_disable();
    return true;
}


bool hk_dsvisible::datasource_enable()
{
#ifdef HK_DEBUG
    hkdebug("dsvisible::datasource_enable");
#endif
    widget_specific_enable();
    widget_specific_enable_disable();
    return true;
}


void hk_dsvisible::insert_mode(void)
{
    p_date.set_now();
    widget_specific_insert_mode();
}


void hk_dsvisible::row_added()
{
    widget_specific_row_added();
}


void hk_dsvisible::row_deleted()
{
    widget_specific_row_deleted();

}


void hk_dsvisible::before_source_vanishes(void)
{
#ifdef HK_DEBUG
    hkdebug("dsvisible::before_source_vanishes");
#endif
    hk_datasource*d =datasource();
    if (d==NULL) return;
    if (d->is_enabled()) d->disable();
    set_datasource(NULL);

}


void    hk_dsvisible::list_changes(filetype type)
{
}


void hk_dsvisible::savedata(ostream& s)
{
#ifdef HK_DEBUG
    hkdebug("dsvisible::savedata");
#endif
    savedata(s,false,false);
}


void hk_dsvisible::savedata(ostream& s, bool savedatasource,bool saveall )
{
#ifdef HK_DEBUG
    hkdebug("dsvisible::savedata(ostream& s, bool savedatasource,bool saveall )");
#endif
    hk_string tag="HK_DSVISIBLE";
    start_mastertag(s,tag);
    set_tagvalue(s,"PRESENTATIONDATASOURCE",p_presentationdatasource);
    set_tagvalue(s,"READONLY",p_readonly);

    if (savedatasource&& datasource()!=NULL)
        datasource()->savedata(s,saveall);

    hk_visible::savedata(s);
    end_mastertag(s,tag);

}


void hk_dsvisible::loaddata(const hk_string& definition)
{
#ifdef HK_DEBUG
    hkdebug("dsvisible::loaddata");
#endif
    get_tagvalue(definition,"PRESENTATIONDATASOURCE",p_presentationdatasource);
    set_presentationdatasource(p_presentationdatasource);
    get_tagvalue(definition,"READONLY",p_readonly);
    hk_string buffer;
    get_tagvalue(definition,"HK_VISIBLE",buffer);
    hk_visible::loaddata(buffer);

    if (p_presentationdatasource!=-1) return;
    hk_string g;

    if (get_tagvalue(definition,"DATASOURCE",g,1,mastertag))
    {
        if (datasource()!=NULL)datasource()->loaddata(g);
    }

}


void    hk_dsvisible::set_readonly(bool readonly)
{
    p_readonly=readonly;

}


bool    hk_dsvisible::is_readonly(void) const
{
    return p_readonly;

}
