// ****************************************************************************
// copyright (c) 2000-2002 Horst Knorr <hk_classes@knoda.org>
// This file is part of the hk_classes library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
//$Revision: 1.50 $

#include "hk_presentation.h"
#include "hk_database.h"
#include "hk_datasource.h"
#include "hk_definitions.h"
#include "hk_dsvisible.h"
#include "hk_form.h"
#include "hk_report.h"
#include <cassert>

hk_presentation::enum_sizetype hk_presentation::p_defaultsizetype=hk_presentation::absolute;

const double teiler=10000.0;

hk_presentation::hk_presentation(void):hk_dsvisible()
{
#ifdef HK_DEBUG
    hkdebug("hk_presentation::hk_presentation");
#endif
    p_database=NULL;
    p_has_changed=false;
    p_automatic_enable_datasources=true;
    p_counter=0;
    p_presentationtype=hk_presentation::general;
    p_vupncentral=0;
    set_mode(hk_presentation::view);
    p_presentation=this;
    p_sizetype=p_defaultsizetype;
    set_designsize(100,100);
}


hk_presentation::~hk_presentation(void)
{
#ifdef HK_DEBUG
    hkdebug("hk_presentation::~hk_presentation");
#endif
    if (p_datasource!=NULL)
    {
        p_datasource->visible_remove(this);
        p_datasource->p_presentation=NULL;
    }
    clear_datasourcelist();
    p_datasource=NULL;
    if (p_database!=NULL) p_database->presentation_remove(this);
}


void hk_presentation::set_database(hk_database* db)
{
#ifdef HK_DEBUG
    hkdebug("hk_presentation::set_database");
#endif

    if (p_database!=NULL)
    {
        p_database->presentation_remove(this);
        p_database=NULL;
    }
    if (db!=NULL)
    {
        db->presentation_add(this);
    }

    p_database=db;

}


long hk_presentation::add_datasource(hk_datasource* d)
{
#ifdef HK_DEBUG
    hkdebug("hk_presentation::add_datasource");
#endif
    if (d==NULL) 
     {
       show_warningmessage(hk_translate("Bug: tried to add empty datasource!"));
       return -1;
     }
    d->p_dscounter=p_counter;
    p_counter++;
    p_datasources.insert(p_datasources.end(),d);
    return d->p_dscounter;
}


long    hk_presentation::new_datasource(const hk_string& name,bool query)
{
#ifdef HK_DEBUG
    hkdebug("hk_presentation::new_datasource");
#endif
    if (p_database==NULL) return -1;
    hk_datasource* d=p_database->load_datasource(name,query,this);
    if (d) d->set_name(name,false);
    long i=add_datasource(d);
#ifdef HK_DEBUG
    if (d)d->hkclassname(longint2string(i));
#endif
    set_has_changed();
    return i;
}


hk_datasource*  hk_presentation::get_datasource(long nr)
{
#ifdef HK_DEBUG
    hkdebug("hk_presentation::get_datasource(long)");
#endif
    list<hk_datasource*>::iterator it=p_datasources.begin();
    while (it!=p_datasources.end())
    {
        if ( (*it)->p_dscounter==nr) return (*it);
        it++;
    }
    return NULL;
}


hk_datasource*  hk_presentation::get_datasource(const hk_string& description)
{
#ifdef HK_DEBUG
    hkdebug("hk_presentation::get_datasource(const hk_string&)");
#endif
    list<hk_datasource*>::iterator it=p_datasources.begin();
    while (it!=p_datasources.end())
    {
        if ( unique_datasourcename((*it)->p_dscounter)==description) 
	  return (*it);
        it++;
    }
    return NULL;
}


void    hk_presentation::remove_datasource(long nr)
{
#ifdef HK_DEBUG
    hkdebug("hk_presentation::remove_datasource");
#endif
    remove_datasource(get_datasource(nr));
}


void hk_presentation::remove_datasource( hk_datasource* d)
{
#ifdef HK_DEBUG
    hkdebug("hk_presentation::remove_datasource");
#endif
    if (d!=NULL)
    {
        p_datasources.remove(d);
        set_has_changed();
    }
}


void hk_presentation::clear_datasourcelist(void)
{
#ifdef HK_DEBUG
    hkdebug("hk_presentation::clear_datasourcelist");
#endif
    list<hk_datasource*>::iterator it=p_datasources.begin();
    while (it!=p_datasources.end())
    {
        hk_datasource* d=(*it);
        it++;
        if (d!=NULL)
        {
            d->store_changed_data();
            d->disable();
            d->p_presentation=NULL;
        }

//     if (d!=nodelete)delete d;
        delete d;
    }

    p_datasources.clear();
#ifdef HK_DEBUG
    hkdebug("hk_presentation::clear_datasourcelist ENDE");
#endif
}


void    hk_presentation::savedata(ostream& s)
{
#ifdef HK_DEBUG
    hkdebug("hk_presentation::savedata");
#endif
    hk_string tag="PRESENTATION";
    start_mastertag(s,tag);
    hk_dsvisible::savedata(s);
    set_tagvalue(s,"DESIGNWIDTH",(long unsigned int)p_designwidth);
    set_tagvalue(s,"DESIGNHEIGHT",(long unsigned int)p_designheight);
    hk_string stype=(p_sizetype==absolute?"ABSOLUTE":"RELATIVE");

    set_tagvalue(s,"SIZETYPE",stype);
    list<hk_datasource*>::iterator it=p_datasources.begin();
    start_mastertag(s,"DATASOURCES");
    if (p_datasources.size()==0) show_warningmessage(hk_translate("savedata No datasources defined!")) ;
    while (it!=p_datasources.end())
    {
        (*it)->savedata(s);
        it++;
    }
    end_mastertag(s,"DATASOURCES");

    end_mastertag(s,tag);
}


void    hk_presentation::loaddata(const hk_string& definition)
{
#ifdef HK_DEBUG
    hkdebug("hk_presentation::loaddata");
#endif
    get_tagvalue(definition,"DESIGNWIDTH", p_designwidth);
    get_tagvalue(definition,"DESIGNHEIGHT",p_designheight);
    set_designsize(p_designwidth,p_designheight,false);
    hk_string buffer2;
    p_sizetype=relative;
    if (get_tagvalue(definition,"SIZETYPE",buffer2))
    {
      if (buffer2=="ABSOLUTE")p_sizetype=absolute;
    }


    hk_string value;
    int i=1;
    clear_datasourcelist();
    while (get_tagvalue(definition,"DATASOURCE",value,i,mastertag))
    {
        hk_datasource* ds=NULL;
        get_tagvalue(value,"TYPE",buffer2);
        hk_string dsname;
        get_tagvalue(value,"DATASOURCENAME",dsname);
        bool pquery=true;
        if (buffer2=="TABLE")  pquery=false;
        long int dint=new_datasource(dsname,pquery);
        ds=get_datasource(dint);
        if (ds!=NULL)
        {
            ds->loaddata(value,!pquery);
	    ds->set_name(dsname,false);
            if (ds->p_dscounter>=p_counter) p_counter=ds->p_dscounter+1;
        } else show_warningmessage(hk_translate("Error creating/loading datasource! This is probably a bug"));
        i++;
    }
    if (p_datasources.size()==0) show_warningmessage(hk_translate("No datasources defined!")) ;
    get_tagvalue(definition,"HK_DSVISIBLE",value);
    hk_dsvisible::loaddata(value);

}


hk_string hk_presentation::unique_datasourcename(long nr)
{
    hk_string result;
    hk_datasource* rd =get_datasource(nr);
    if (rd!=NULL)
    {
        result=rd->name()+" (";
        result+=longint2string(rd->p_dscounter);
    }
    result+=") ";
    hk_datasource* depds=get_datasource(rd->depending_on_presentationdatasource());
    if (depds!=NULL)
    {
        result+=" [";
        result+=depds->name()+"]";

    }
    if (rd->filter().size()>0)
    {
        result+=" | ";
        result+=rd->filter();
    }
    return result;
}


void hk_presentation::set_has_changed(void)
{
#ifdef HK_DEBUG
    hkdebug("hk_presentation::set_has_changed");
#endif
    if (p_mode==design)
        p_has_changed=true;
}


bool hk_presentation::has_changed(void)
{
    return p_has_changed;
}


bool hk_presentation::set_mode(enum_mode s)
{

    if (s==hk_presentation::design && runtime_only()) return false;
    if (!widget_specific_before_modechanges(s)) return false;
    p_mode=s;
    if (p_automatic_enable_datasources)
    {
        bool enable=(s==hk_presentation::view);
        enable_datasources(enable,hk_datasource::standard);
    }
    widget_specific_modechanges(s);
    return true;
}


void hk_presentation::enable_datasources(bool enable,hk_datasource::enum_accessmode mode)
{
    list<hk_datasource*>* mydatasources=datasources();
    list<hk_datasource*>::iterator it = mydatasources->begin();
    while (it!=mydatasources->end())
    {
        if (enable)
        {
            if ((*it)->datasource_used())
            {
                if ((*it)->is_enabled()) (*it)->disable();
                (*it)->set_accessmode(mode);
                (*it)->enable();
            }
        }
        else
        {
            (*it)->disable();
            (*it)->set_accessmode(mode);
        }
        it++;
    }

}


bool hk_presentation::widget_specific_before_modechanges(enum_mode)
{
    return true;
}


void hk_presentation::reset_has_changed(void)
{
    p_has_changed=false;
}


void hk_presentation::database_delete(void)
{
    database_has_vanished();
    p_database =NULL;
}


void hk_presentation::database_has_vanished(void)
{

}


void hk_presentation::before_source_vanishes(void)
{
    
    //hk_dsvisible::before_source_vanishes();
    //p_datasources.remove(datasource());
}


bool hk_presentation::ask_name(void)
{
#ifdef HK_DEBUG
    hkdebug("hk_presentation::ask_name");
#endif
    hk_string text;
    if (p_visibletype==hk_visible::form) text=hk_translate("Please enter the name of the form:");
    else text=hk_translate("Please enter the name of the report");
    hk_string res=show_stringvaluedialog(text);
    if (res.size()==0) return false;

    set_name(res);

    return true;

}


void    hk_presentation::has_changed(bool registerchange)
{
    if (registerchange) set_has_changed();
}


void    hk_presentation::set_designsize(unsigned int w, unsigned int h,bool registerchange)
{
#ifdef HK_DEBUG
    hkdebug("hk_presentation::set_designsize");
#endif
    p_designwidth=w;
    p_designheight=h;
    set_presentationsize(w,h);
    if (registerchange) set_has_changed();
}


void    hk_presentation::set_presentationsize(unsigned int w, unsigned int h)
{
#ifdef HK_DEBUG
    hkdebug("hk_presentation::set_presentationsize");
#endif
    p_actualwidth= (w>0?w:1);
    p_actualheight=(h>0?h:1);

    widget_specific_presentationresize();

}


unsigned int    hk_presentation::horizontal2relativ(unsigned int h)
{
#ifdef HK_DEBUG
    hkdebug("hk_presentation::horizontal2relativ");
#endif
    assert(p_designwidth!=0);
    return (unsigned int)( h*teiler / p_designwidth+0.5) ;

}


unsigned int    hk_presentation::vertical2relativ(unsigned int v)
{
#ifdef HK_DEBUG
    hkdebug("hk_presentation::vertical2relativ");
#endif
    assert(p_designheight!=0);
    return (unsigned int)(v*teiler / p_designheight+0.5);

}


unsigned int    hk_presentation::relativ2horizontal(unsigned int r)
{
#ifdef HK_DEBUG
    hkdebug("hk_presentation::relativ2horizontal");
#endif
    return (unsigned int)(r*p_designwidth /teiler+0.5);

}


unsigned int    hk_presentation::relativ2vertical(unsigned int r)
{
#ifdef HK_DEBUG
    hkdebug("hk_presentation::relativ2vertical");
#endif
    return (unsigned int)(r*p_designheight/teiler+0.5);

}


void hk_presentation::register_object(hk_visible* v)
{
    if (v==NULL) return;
    if (v->p_vupn==-1)
    {
        v->p_vupn=p_vupncentral;
        p_vupncentral++;
    }
    else
    {
        if (v->p_vupn>=p_vupncentral)
            p_vupncentral=v->p_vupn+1;
    }

}


unsigned int    hk_presentation::designwidth(void)
{
    if (type()==hk_visible::report)
    {
        hk_report* r =dynamic_cast <hk_report*>(this);
        if (r!=NULL)
        {
            if (r->masterreport()!=NULL) return r->masterreport()->designwidth();
        }

    }

    return p_designwidth;
}


unsigned int    hk_presentation::designheight(void)
{
    if (type()==hk_visible::report)
    {
        hk_report* r =dynamic_cast <hk_report*>(this);
        if (r!=NULL)
        {
            if (r->masterreport()!=NULL) return r->masterreport()->designheight();
        }

    }
    return p_designheight;
}


void hk_presentation::set_automatic_enable_datasources(bool b)
{
    p_automatic_enable_datasources=b;
}


bool hk_presentation::automatic_enable_datasources(void)
{
    return p_automatic_enable_datasources;
}


void hk_presentation::bulk_operation(enum_bulkoperation )
{

}



void hk_presentation::set_sizetype(enum_sizetype type,bool registerchange)
{
  if (type==p_sizetype) return;
  p_sizetype=type;
  has_changed(registerchange);
  sizetype_changed();

}



hk_presentation::enum_sizetype hk_presentation::sizetype(void)
{
   return p_sizetype;
}


void hk_presentation::sizetype_changed(void)
{


}


 void hk_presentation::set_defaultsizetype(enum_sizetype type)
{
 p_defaultsizetype=type;

}


hk_presentation::enum_sizetype hk_presentation::defaultsizetype(void)
 {
 
 return p_defaultsizetype;
 }
 
