// ****************************************************************************
// copyright (c) 2000-2002 Horst Knorr <hk_classes@knoda.org>
// This file is part of the hk_classes library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
//$Revision: 1.40 $
#include "hk_reportdata.h"
#include "hk_reportsection.h"
#include "hk_report.h"
#include <math.h>
#include "hk_reportutils.h"
#include "hk_column.h"
#include "hk_datasource.h"

hk_reportdata::reportdataconfigurelisttype  hk_reportdata::p_reportdataconfigurefunctions;
list<hk_string> hk_reportdata::p_reportdataconfigurelist;

hk_reportdata::datacountlisttype  hk_reportdata::p_datacountfunctions;
list<hk_string> hk_reportdata::p_datacountfunctionlist;

hk_reportdata::datareplacelisttype  hk_reportdata::p_datareplacefunctions;
list<hk_string> hk_reportdata::p_datareplacefunctionlist;

//********************************************
//**   hk_reportdata                        **
//********************************************

hk_reportdata::hk_reportdata(hk_reportsection* s):hk_dsdatavisible(s->report())
{
#ifdef HK_DEBUG
    hkdebug("hk_reportdata::hk_reportdata");
#endif

    p_visibletype=hk_visible::reportdata;
    p_report=s!=NULL?s->report():NULL;
    set_backgroundcolour(p_report?p_report->backgroundcolour():hk_white,false);
    set_foregroundcolour(p_report?p_report->foregroundcolour():hk_black,false);
    p_section=s;
    p_count=0;
//  commadigits()=-1;
    p_runningcount=false;
    p_minmax_alreadyset=false;
    p_replacefunction=NULL;
    p_datacountfunction=NULL;
    p_top=p_bottom=p_left=p_right=p_luro=p_loru=false;
    p_wordbreak=false;
    set_height(40,false);
    set_width(300,false);
//  wanna_debug(true);
    if (p_reportdataconfigurefunctions.size()==0)
    {
        add_configurefunctiontype("Postscript",&configure_postscriptdata);
        add_configurefunctiontype("None",NULL);
    }

    set_configurefunction("None",false);

    if (p_datacountfunctions.size()==0)
    {
        add_datacountfunctiontype("None",NULL);
    }
    set_datacountfunction("None",false);

    if (p_datareplacefunctions.size()==0)
    {
        add_datareplacefunctiontype("None",NULL);
    }
    set_replacefunction("None",false);
    set_data("%VALUE%",false);

}


hk_reportdata::~hk_reportdata()
{
#ifdef HK_DEBUG
    hkdebug("hk_reportdata::~hk_reportdata");
#endif
    if (p_section!=NULL) p_section->remove_data(this);
}


void hk_reportdata::set_data(const hk_string& d,bool registerchange)
{
    p_data=d;
    has_changed(registerchange);
}


hk_string hk_reportdata::data(void)
{
    return p_data;
}


void hk_reportdata::set_beforedata(const hk_string& b,bool registerchange)
{
    p_beforedata=b;
    has_changed(registerchange);
}


void hk_reportdata::set_afterdata(const hk_string& a,bool registerchange)
{
    p_afterdata=a;
    has_changed(registerchange);
}


hk_string hk_reportdata::replace(const hk_string& where)
{
    hk_string p_localbuffer;
#ifdef HK_DEBUG
    hkdebug("hk_reportdata::replace");
#endif
    p_localbuffer=hk_dsdatavisible::replace(where);
    list<hk_column*>* cols=p_report->datasource()->columns();
    list<hk_column*>::iterator it=cols->begin();
    while(it!=cols->end())
    {
        hk_string name="#";
        name+=(*it)->name();
        name+="#";
        if (p_report->recodefunction()==NULL)
            p_localbuffer=replace_all(name,p_localbuffer,(*it)->asstring(true));
        else     p_localbuffer=replace_all(name,p_localbuffer,p_report->recodefunction()((*it)->asstring(true),p_report));
        it++;
    }
    p_localbuffer=replace_all("%XPOS%",p_localbuffer,p_report->sizetype()==hk_presentation::relative?longint2string(p_section->relativ2horizontal(x())):longint2string(x()));
    p_localbuffer=replace_all("%YPOS%",p_localbuffer,p_report->sizetype()==hk_presentation::relative?longint2string(p_section->relativ2vertical(y())):longint2string(y()));
    p_localbuffer=replace_all("%WIDTH%",p_localbuffer,p_report->sizetype()==hk_presentation::relative?longint2string(p_section->relativ2horizontal(width())):longint2string(width()));
    p_localbuffer=replace_all("%HEIGHT%",p_localbuffer,p_report->sizetype()==hk_presentation::relative?longint2string(p_section->relativ2vertical(height())):longint2string(height()));
    p_localbuffer=replace_all("%RELXPOS%",p_localbuffer,p_report->sizetype()==hk_presentation::relative?longint2string(x()):longint2string(p_section->horizontal2relativ(x())));
    p_localbuffer=replace_all("%RELYPOS%",p_localbuffer,p_report->sizetype()==hk_presentation::relative?longint2string(y()):longint2string(p_section->vertical2relativ(y())));
    p_localbuffer=replace_all("%RELWIDTH%",p_localbuffer,p_report->sizetype()==hk_presentation::relative?longint2string(width()):longint2string(p_section->horizontal2relativ(width())));
    p_localbuffer=replace_all("%RELHEIGHT%",p_localbuffer,p_report->sizetype()==hk_presentation::relative?longint2string(height()):longint2string(p_section->vertical2relativ(height())));
    p_localbuffer=replace_all("%PAGENUMBER%",p_localbuffer,format_number(p_report->pagenumber(),use_numberseparator(),(commadigits()<0?0:commadigits())));
    p_localbuffer=replace_all("%ABSOLUTEPAGENUMBER%",p_localbuffer,format_number(p_report->absolutepagenumber(),use_numberseparator(),(commadigits()<0?0:commadigits())));
    p_localbuffer=replace_all("%ROWNUMBER%",p_localbuffer,format_number(p_report->rownumber(),use_numberseparator(),(commadigits()<0?0:commadigits())));

    hk_string align;
    switch (alignment())
    {
        case alignright : align="RIGHT";break;
        case aligncenter : align="CENTER";break;
        default         : align="LEFT";
    };

    p_localbuffer=replace_all("%ALIGN%",p_localbuffer,align);

    if (column()!=NULL)
    {
        p_localbuffer=replace_all("%value%",p_localbuffer,column()->asstring(true));
        p_localbuffer=replace_all("%fieldname%",p_localbuffer,columnname());
        p_localbuffer=replace_all("%displayname%",p_localbuffer,p_displayname);
        p_localbuffer=replace_all("%COUNT%",p_localbuffer,format_number(p_count,use_numberseparator(),(commadigits()<0?0:commadigits())));
        hk_string format="%.3f";
        switch  (column()->columntype())
        {
            case  hk_column::integercolumn: ;
            case hk_column::auto_inccolumn: ;
            case hk_column::smallintegercolumn :
            {
                p_localbuffer=replace_all("%SUM%",p_localbuffer,format_number(sum.integer,use_numberseparator(),(commadigits()<0?0:commadigits())));
                p_localbuffer=replace_all("%MIN%",p_localbuffer,format_number(min.integer,use_numberseparator(),(commadigits()<0?0:commadigits())));
                p_localbuffer=replace_all("%MAX%",p_localbuffer,format_number(max.integer,use_numberseparator(),(commadigits()<0?0:commadigits())));
                if (p_count>0)
                {
                    double average=(double)sum.integer/(double)p_count;
                    p_localbuffer=replace_all("%AVERAGE%",p_localbuffer,format_number(average,use_numberseparator(),commadigits()));
                    double stddev=sqrt((squaresum.integer-2*average*sum.integer+p_count*average*average)/p_count);
                    p_localbuffer=replace_all("%STDDEV%",p_localbuffer,format_number(stddev,use_numberseparator(),commadigits()));
                    stddev=sqrt((squaresum.integer-2*average*sum.integer+p_count*average*average)/(p_count-1));
                    p_localbuffer=replace_all("%STDDEVSAMPLE%",p_localbuffer,format_number(stddev,use_numberseparator(),commadigits()));
                }
            };break;

            case hk_column::floatingcolumn:  ;
            case hk_column::smallfloatingcolumn:
            {
                p_localbuffer=replace_all("%SUM%",p_localbuffer,format_number(sum.real,use_numberseparator(),commadigits()));
                p_localbuffer=replace_all("%MIN%",p_localbuffer,format_number(min.real,use_numberseparator(),commadigits()));
                p_localbuffer=replace_all("%MAX%",p_localbuffer,format_number(max.real,use_numberseparator(),commadigits()));
                if (p_count>0)
                {
                    double average=sum.real/(double)p_count;
                    p_localbuffer=replace_all("%AVERAGE%",p_localbuffer,format_number(average,use_numberseparator(),commadigits()));
                    double stddev=sqrt((squaresum.real-2*average*sum.real+p_count*average*average)/p_count);
                    p_localbuffer=replace_all("%STDDEV%",p_localbuffer,format_number(stddev,use_numberseparator(),commadigits()));
                    stddev=sqrt((squaresum.real-2*average*sum.real+p_count*average*average)/(p_count-1));
                    p_localbuffer=replace_all("%STDDEVSAMPLE%",p_localbuffer,format_number(stddev,use_numberseparator(),commadigits()));

                }
                const char* i="?";
                p_localbuffer=replace_all("%AVERAGE%",p_localbuffer,i);
                p_localbuffer=replace_all("%STDDEV%",p_localbuffer,i);
                p_localbuffer=replace_all("%STDDEVSAMPLE%",p_localbuffer,i);
            } break;
            default:;

        }

        if (p_report!=NULL)
        {
            recodefunctiontype* r=p_report->recodefunction();
            hk_string value;
            hk_string name;
            hk_string display;

            if (is_numerictype(column()))
            {
                double n=atof(column()->asstring(true).c_str());
                value=format_number(n,use_numberseparator(),is_integertype(column())?0:(commadigits()<0)?-1:commadigits());
            }
            else   value=column()->asstring(true);
            name=columnname();
            display=p_displayname;

            if (r!=NULL)
            {
                value=r(value,p_report);
                name=r(name,p_report);
                display=r(display,p_report);
            }
            p_localbuffer=replace_all("%VALUE%",p_localbuffer,value);
            p_localbuffer=replace_all("%FIELDNAME%",p_localbuffer,name);
            p_localbuffer=replace_all("%DISPLAYNAME%",p_localbuffer,display);
        }

    }
    p_localbuffer=replace_all("%FONT%",p_localbuffer,font().fontname());
    p_localbuffer=replace_all("%PSFONT%",p_localbuffer,font().psfontname());
    p_localbuffer=replace_all("%FONTSIZE%",p_localbuffer,longint2string(font().fontsize()));
    return p_localbuffer;

}


hk_string hk_reportdata::actual_string(void)
{
#ifdef HK_DEBUG
    hkdebug("hk_reportdata::actual_string");
#endif
    hk_string r;
    hk_string b;
    r=p_beforedata+p_data+p_afterdata;
    r=replace(r);
    b=r;
    if (p_replacefunction!=NULL) r=p_replacefunction(this,r);
    if (p_datacountfunction!=NULL)set_counts_as(p_datacountfunction(this));
    p_report->rowcount(counts_as());
    r=b;
    return  r;

}


void hk_reportdata::count(void)
{
#ifdef HK_DEBUG
    hkdebug("hk_reportdata::count");
#endif
    p_count++;
    if(column()==0) return;

    switch(column()->columntype())
    {
        case hk_column::integercolumn  :;
        case hk_column::auto_inccolumn :;
        case hk_column::smallintegercolumn:
        {
            hk_string s=column()->asstring(true);
            int r=atoi(s.c_str());
            if (datasource()->row_position()==0)
            {
                sum.integer=r;
                squaresum.integer=r*r;
            }
            else
            {
                sum.integer+=r;
                squaresum.integer+=r*r;
            }
            if (s.size()>0)
            {
                if (p_minmax_alreadyset)
                {
                    if (r<min.integer)min.integer=r;
                    if (r>max.integer)max.integer=r;
                }
                else
                {
                    min.integer=r;
                    max.integer=r;
                    p_minmax_alreadyset=true;
                }
            }

        };break;
        case  hk_column::floatingcolumn:;
        case  hk_column::smallfloatingcolumn:;
        {
            hk_string s=column()->asstring(true);
            double r=strtod(s.c_str(),NULL);
            if (datasource()->row_position()==0)
            {
                sum.real=r;
                squaresum.real=r*r;
            }
            else
            {
                sum.real+=r;
                squaresum.real+=r*r;
            }
            if (s.size()>0)
            {
                if (p_minmax_alreadyset)
                {
                    if (r<min.real)min.real=r;
                    if (r>max.real)max.real=r;
                }
                else
                {
                    min.real=r;
                    max.real=r;
                    p_minmax_alreadyset=true;
                }
            }
        }
        default:;
    }
}


void hk_reportdata::set_runningcount(bool c,bool registerchange)
{
    p_runningcount=c;
    has_changed(registerchange);
}


bool hk_reportdata::runningcount(void)
{

    return p_runningcount;
}


void hk_reportdata::set_replacefunction(const hk_string& f,bool registerchange)
{
#ifdef HK_DEBUG
    hkdebug("hk_reportdata::set_replacefunction");
#endif
    if (f==p_replacefunctionstring)return;
    datareplacelisttype::iterator it=p_datareplacefunctions.find(f);
    if (it==p_datareplacefunctions.end())
    {

        show_warningmessage(hk_translate("replacefunction not found"));
        p_replacefunction=NULL;
        p_replacefunctionstring="None";
        return;
    }
    data_replacefunctiontype* c=*it->second;

    p_replacefunction=c;
    p_replacefunctionstring=f;
    has_changed(registerchange);
}


void hk_reportdata::set_datacountfunction(const hk_string& f,bool registerchange)
{
#ifdef HK_DEBUG
    hkdebug("hk_reportdata::set_datacountfunction");
#endif
    if (f==p_datacountfunctionstring)return;

    datacountlisttype::iterator it=p_datacountfunctions.find(f);
    if (it==p_datacountfunctions.end())
    {

        show_warningmessage(hk_translate("Data Countfunction not found"));
        p_datacountfunction=NULL;
        p_datacountfunctionstring="None";
        return;
    }
    reportdatacounttype* c=*it->second;

    p_datacountfunction=c;
    p_datacountfunctionstring=f;
    has_changed(registerchange);
}


hk_string hk_reportdata::datacountfunctionstring(void)
{
    return p_datacountfunctionstring;

}


reportdatacounttype* hk_reportdata::datacountfunction(void)
{

    return p_datacountfunction;
}


bool    hk_reportdata::diagonalloru(void)
{
    return p_loru;
}


bool    hk_reportdata::topline(void)
{
    return p_top;
}


bool    hk_reportdata::bottomline(void)
{
    return p_bottom;
}


bool    hk_reportdata::leftline(void)
{
    return p_left;
}


bool    hk_reportdata::rightline(void)
{
    return p_right;
}


bool    hk_reportdata::diagonalluro(void)
{
    return p_luro;
}


bool    hk_reportdata::wordbreak(void)
{
    return p_wordbreak;
}


void hk_reportdata::reset_count()
{
#ifdef HK_DEBUG
    hkdebug("hk_reportdata::reset_count");
#endif
    if (!p_runningcount)
    {
        p_count=0;
        if (column()!=NULL)
            if (column()->columntype()==hk_column::integercolumn||
            column()->columntype()==hk_column::smallintegercolumn||
            column()->columntype()==hk_column::auto_inccolumn)
        {
            sum.integer=0;
            squaresum.integer=0;
            min.integer=0;
            max.integer=0;
            p_minmax_alreadyset=false;
        }
        else
        if (column()->columntype()==hk_column::floatingcolumn||
            column()->columntype()==hk_column::smallfloatingcolumn)
        {
            sum.real=0;
            squaresum.real=0;
            min.real=0;
            max.real=0;
            p_minmax_alreadyset=false;
        }
    }
}


hk_font hk_reportdata::font(void)
{
#ifdef HK_DEBUG
    hkdebug("hk_reportdata::font");
#endif
    if (hk_dsdatavisible::font().fontname().size()==0) return p_section->font();
    else return hk_dsdatavisible::font();
}




void    hk_reportdata::set_topline(bool l,bool registerchange)
{
    p_top=l;
    if(p_dataconfigurefunction!=NULL)p_dataconfigurefunction(this);
    has_changed(registerchange);
}


void    hk_reportdata::set_bottomline(bool l,bool registerchange)
{
    p_bottom=l;
    if(p_dataconfigurefunction!=NULL)p_dataconfigurefunction(this);
    has_changed(registerchange);

}


void    hk_reportdata::set_leftline(bool l,bool registerchange)
{
    p_left=l;
    if(p_dataconfigurefunction!=NULL)p_dataconfigurefunction(this);
    has_changed(registerchange);
}


void    hk_reportdata::set_rightline(bool l,bool registerchange)
{
    p_right=l;
    has_changed(registerchange);
    if(p_dataconfigurefunction!=NULL)p_dataconfigurefunction(this);

}


void    hk_reportdata::set_diagonalluro(bool l,bool registerchange)
{
    p_luro=l;
    if(p_dataconfigurefunction!=NULL)p_dataconfigurefunction(this);
    has_changed(registerchange);
}


void    hk_reportdata::set_diagonalloru(bool l,bool registerchange)
{
    p_loru=l;
    if(p_dataconfigurefunction!=NULL)p_dataconfigurefunction(this);
    has_changed(registerchange);
}


void     hk_reportdata::set_frame(bool l,bool registerchange)
{
#ifdef HK_DEBUG
    hkdebug("hk_reportdata::set_frame");
#endif
    p_top=l;
    p_bottom=l;
    p_left=l;
    p_right=l;
    if(p_dataconfigurefunction!=NULL)p_dataconfigurefunction(this);
    has_changed(registerchange);
}


void hk_reportdata::set_wordbreak(bool b,bool registerchange)
{
    p_wordbreak=b;
    if(p_dataconfigurefunction!=NULL)p_dataconfigurefunction(this);
    has_changed(registerchange);
}


void  hk_reportdata::savedata(ostream& s,bool userdefined)
{
#ifdef HK_DEBUG
    hkdebug("hk_reportdata::savedata");
#endif
    start_mastertag(s,"HK_REPORTDATA");
    hk_dsdatavisible::savedata(s);
    set_tagvalue(s,"DATAVALUE",p_data);
    set_tagvalue(s,"DISPLAYNAME",p_displayname);
    set_tagvalue(s,"TOPBORDER",p_top);
    set_tagvalue(s,"LEFTBORDER",p_left);
    set_tagvalue(s,"RIGHTBORDER",p_right);
    set_tagvalue(s,"BOTTOMBORDER",p_bottom);
    set_tagvalue(s,"DIAGONALLORU",p_loru);
    set_tagvalue(s,"DIAGONALLURO",p_luro);
    set_tagvalue(s,"WORDBREAK",p_wordbreak);
    set_tagvalue(s,"RUNNINGCOUNT",p_runningcount);

    if (userdefined)
    {
        set_tagvalue(s,"BEFOREDATA",p_beforedata);
        set_tagvalue(s,"AFTERDATA",p_afterdata);
        set_tagvalue(s,"DATACONFIGUREFUNCTION",p_dataconfigurefunctionstring);
        set_tagvalue(s,"DATACOUNTFUNCTION",p_datacountfunctionstring);
        set_tagvalue(s,"DATAREPLACEFUNCTION",p_replacefunctionstring);
    }

    end_mastertag(s,"HK_REPORTDATA");

}


void  hk_reportdata::loaddata(const hk_string& definition,bool userdefined)
{
#ifdef HK_DEBUG
    hkdebug("hk_reportdata::loaddata");
#endif
    hk_dsdatavisible::loaddata(definition);
    get_tagvalue(definition,"DATAVALUE",p_data);

    hk_string sbuffer;
    if (userdefined)
    {
        get_tagvalue(definition,"BEFOREDATA",p_beforedata);
        get_tagvalue(definition,"AFTERDATA",p_afterdata);
        if (get_tagvalue(definition,"DATACONFIGUREFUNCTION",sbuffer))
            set_configurefunction(sbuffer);
        if (get_tagvalue(definition,"DATACOUNTFUNCTION",sbuffer))
            set_datacountfunction(sbuffer,false);
        if (get_tagvalue(definition,"DATAREPLACEFUNCTION",sbuffer))
            set_replacefunction(sbuffer);

    }
    get_tagvalue(definition,"DISPLAYNAME",p_displayname);
    get_tagvalue(definition,"TOPBORDER",p_top);
    get_tagvalue(definition,"LEFTBORDER",p_left);
    get_tagvalue(definition,"RIGHTBORDER",p_right);
    get_tagvalue(definition,"BOTTOMBORDER",p_bottom);
    get_tagvalue(definition,"DIAGONALLORU",p_loru);
    get_tagvalue(definition,"DIAGONALLURO",p_luro);
    get_tagvalue(definition,"WORDBREAK",p_wordbreak);
    get_tagvalue(definition,"RUNNINGCOUNT",p_runningcount);

    hk_string configurefunc;
    get_tagvalue(definition,"DATACONFIGUREFUNCTION",configurefunc);
    if  (configurefunc=="POSTSCRIPT") p_dataconfigurefunction=&configure_postscriptdata;
    if(p_dataconfigurefunction!=NULL)p_dataconfigurefunction(this);

}


void    hk_reportdata::set_configurefunction(const hk_string& f,bool registerchange)
{
#ifdef HK_DEBUG
    hkdebug("hk_reportdata::set_configurefunction");
#endif

    if (f==p_dataconfigurefunctionstring)return;

    reportdataconfigurelisttype::iterator it=p_reportdataconfigurefunctions.find(f);
    if (it==p_reportdataconfigurefunctions.end())
    {

        show_warningmessage(hk_translate("Dataconfigurefunction not found"));
        p_dataconfigurefunction=NULL;
        p_dataconfigurefunctionstring="None";
        return;
    }

    data_configurefunctiontype* rf=*it->second;
    p_dataconfigurefunction=rf;
    p_dataconfigurefunctionstring=f;
    if (p_dataconfigurefunction!=NULL) p_dataconfigurefunction(this);
    has_changed(registerchange);
}


data_configurefunctiontype* hk_reportdata::configurefunction(void)
{
    return p_dataconfigurefunction;
}


hk_string hk_reportdata::configurefunctionstring(void)
{
    return p_dataconfigurefunctionstring;
}


void    hk_reportdata::neutralize_definition(bool registerchange)
{
#ifdef HK_DEBUG
    hkdebug("hk_reportdata::neutralize_definition");
#endif
    set_beforedata("",registerchange);
    set_afterdata("",registerchange);
    set_replacefunction("None",registerchange);
    set_datacountfunction("None",registerchange);
    set_configurefunction("None",registerchange);

}


void hk_reportdata::add_configurefunctiontype(const hk_string& name,data_configurefunctiontype* f)
{
    if (name.size()==0) return;
    typedef reportdataconfigurelisttype::value_type recodevalue;
    p_reportdataconfigurefunctions.insert(recodevalue(name,f));
    p_reportdataconfigurelist.insert(p_reportdataconfigurelist.end(),name);

}


list<hk_string>* hk_reportdata::configurefunctionlist(void)
{
    return &p_reportdataconfigurelist;
}


void hk_reportdata::add_datacountfunctiontype(const hk_string& name,reportdatacounttype* f)
{

    if (name.size()==0) return;
    typedef datacountlisttype::value_type datacountvalue;
    p_datacountfunctions.insert(datacountvalue(name,f));
    p_datacountfunctionlist.insert(p_datacountfunctionlist.end(),name);
}


list<hk_string>* hk_reportdata::datacountfunctionlist(void)
{
    return &p_datacountfunctionlist;
}


void hk_reportdata::add_datareplacefunctiontype(const hk_string& name,data_replacefunctiontype* f)
{
    if (name.size()==0) return;
    typedef datareplacelisttype::value_type datareplacevalue;
    p_datareplacefunctions.insert(datareplacevalue(name,f));
    p_datareplacefunctionlist.insert(p_datareplacefunctionlist.end(),name);
}


list<hk_string>* hk_reportdata::datareplacefunctionlist(void)
{
    return &p_datareplacefunctionlist;
}


hk_report* hk_reportdata::report(void)
{
    return p_report;
}


hk_reportsection* hk_reportdata::section(void)
{
    return p_section;
}


void hk_reportdata::new_column_pointer_created(void)
{
    hk_column* col=column();
    if (col)
    {
        if (is_integertype(col)) {sum.integer=0;min.integer=0;max.integer=0;squaresum.integer=0;}
        else if (is_realtype(col)){sum.real=0;min.real=0;max.real=0;squaresum.real=0;}
    }

}


void hk_reportdata::lower_widget(bool registerchange)
{
    if (p_section) p_section->lower_widget(this);
    has_changed(registerchange);
    widget_specific_lower_widget();
}


void hk_reportdata::raise_widget(bool registerchange)
{
    if (p_section) p_section->raise_widget(this);
    has_changed(registerchange);
    widget_specific_raise_widget();

}


void hk_reportdata::sizetype_changed(void)
{
 if (!p_report) return;
 
 if (p_report->sizetype()==hk_presentation::relative)
  {
// this means previous sizetype was hk_presentation::absolute
// and changed now to hk_presentation::relative
   p_x=p_section->horizontal2relativ(p_x);
   p_y=p_section->vertical2relativ(p_y);
   p_width=p_section->horizontal2relativ(p_width);
   p_height=p_section->vertical2relativ(p_height);
  }
else
  {
// this means previous sizetype was hk_presentation::relative
// and changed now to hk_presentation::absolute
   p_x=p_section->relativ2horizontal(p_x);
   p_y=p_section->relativ2vertical(p_y);
   p_width=p_section->relativ2horizontal(p_width);
   p_height=p_section->relativ2vertical(p_height);

  }
}
