// ****************************************************************************
// copyright (c) 2000-2002 Horst Knorr <hk_classes@knoda.org>
// This file is part of the hk_classes library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
//$Revision: 1.53 $
#include "hk_visible.h"
#include "hk_presentation.h"
#include "hk_form.h"
#include "hk_report.h"
#include "hk_datetime.h"
hk_visible::alignmenttype hk_visible::p_defaulttextalign=hk_visible::alignleft;
hk_visible::alignmenttype hk_visible::p_defaultnumberalign=hk_visible::alignright;
bool hk_visible::p_open_maximized_windows=false;

hk_visible::hk_visible(hk_presentation* p):hk_class()
{
#ifdef HK_DEBUG
    hkdebug("hk_visible::hk_visible");
#endif
    p_visibletype=other;
    p_presentation=p;
    p_setcoordinates=p_setwidgetcoordinates=false;
    set_size(0,0,1000,300,false);
    p_neverregisterchange=false;
    p_vupn=-1;
    p_counts_as=0;
    p_dateformat=p_defaultdateformat;
    p_timeformat=p_defaulttimeformat;
    p_datetimeformat=p_defaultdatetimeformat;
    p_load_presentationnumber=true;
    set_alignment(alignleft,false);
    p_foregroundcolour=hk_black;
    p_backgroundcolour=hk_grey;
    
}


hk_visible::~hk_visible(void)
{
#ifdef HK_DEBUG
    hkdebug("hk_visible::destructor");
#endif
    if (p_presentation==NULL) return;

    hk_form* f=dynamic_cast<hk_form*>(p_presentation);
    if (f!=NULL)
    {
        f->remove_visible(this);
    }

}


void hk_visible::set_size(unsigned int x, unsigned int y,unsigned int w, unsigned int h, bool registerchange)
{
#ifdef HK_DEBUG
    hkdebug("hk_visible:set_size");
#endif
    if (p_setcoordinates==true)
    {
#ifdef HK_DEBUG
        hkdebug("hk_visible:set_size psetcoordinates== true !!!");
#endif
        return;
    }

    p_setcoordinates=true;
    unsigned int ppx=x;
    unsigned int ppy=y;
    unsigned int ppw=w;
    unsigned int pph=h;
    unsigned int usex=x;
    unsigned int usey=y;
    unsigned int usew=w;
    unsigned int useh=h;
    if (p_presentation!=NULL&&p_presentation->sizetype()==hk_presentation::relative)
    {
        if (ppw<100)
        {
            ppw=100;                              //cout <<"ppwminanpassung"<<endl;
        }
        if (pph<100)
        {
            pph=100;                              //cout <<"pphminanpassung"<<endl;
        }
        if (ppx>9950)
        {
            ppx=9950;                             //cout <<"ppxmaxanpassung"<<endl;
        }
        if (ppx+ppw>10000)
        {
            ppw=10000-ppx;                        //cout <<"ppwmaxanpassung"<<endl;
        }
        if (ppy>9950)
        {
            ppy=9950;                             //cout <<"ppymaxanpassung"<<endl;
        }
        if (ppy+pph>10000)
        {
            pph=10000-ppy;                        //cout <<"pphmaxanpassung"<<endl;
        }
        usex=ppx;
        usey=ppy;
        usew=ppw;
        useh=pph;
        ppx=p_presentation->relativ2horizontal(ppx);
        ppy=p_presentation->relativ2vertical(ppy);
        pph=p_presentation->relativ2vertical(pph);
        ppw=p_presentation->relativ2horizontal(ppw);

    }
    if (p_setwidgetcoordinates)
    {
#ifdef HK_DEBUG
        hkdebug("hk_visible:set_size p_setwidgetcoordinates ==true");
#endif
        p_x=usex;
        p_y=usey;
        p_width=usew;
        p_height=useh;
    }
    else
    if(widget_specific_coordinates(ppx,ppy,ppw,pph))
    {
        p_x=usex;
        p_y=usey;
        p_width=usew;
        p_height=useh;
        if (p_presentation!=NULL) p_presentation->widget_specific_fieldresize(this);
    }
    has_changed(registerchange);
    p_setcoordinates=false;
}


void hk_visible::set_x(unsigned int px, bool registerchange)
{
    set_size(px,p_y,p_width,p_height,registerchange);
}


void hk_visible::set_y(unsigned int py, bool registerchange)
{
    set_size(p_x,py,p_width,p_height,registerchange);
}


void hk_visible::set_width(unsigned int pwidth, bool registerchange)
{
    set_size(p_x,p_y,pwidth,p_height,registerchange);
}


void hk_visible::set_size(unsigned int width, unsigned int height, bool registerchange)
{
    set_size(p_x,p_y,width,height,registerchange);
}


void hk_visible::set_position(unsigned int x, unsigned int y, bool registerchange)
{
    set_size(x,y,p_width,p_height,registerchange);
}


void hk_visible::set_height(unsigned int pheight, bool registerchange)
{
    set_size(p_x,p_y,p_width,pheight,registerchange);
}


unsigned int hk_visible::x() const
{
    return p_x;
}


unsigned int hk_visible::y() const
{
    return p_y;
}


unsigned int hk_visible::width() const
{
    return p_width;
}


unsigned int hk_visible::height() const
{
    return p_height;
}


bool    hk_visible::widget_specific_coordinates(unsigned int px,unsigned int py,unsigned int pwidth,unsigned int pheight)
{
    return true;
}


void    hk_visible::widget_specific_font_changed(void)
{

}


void  hk_visible::widget_specific_form_resizes(void)
{
    set_size(p_x,p_y,p_width,p_height);
}




void hk_visible::set_label(const hk_string& l, bool registerchange)
{
#ifdef HK_DEBUG
    hkdebug("hk_visible::set_label(hk_string)");
#endif
    p_label=l;
    has_changed(registerchange);
    widget_specific_label_changed();
}


hk_string hk_visible::label(void)
{

    return p_label;
}


void hk_visible::savedata(ostream& s )
{
#ifdef HK_DEBUG
    hkdebug("hk_visible::savedata");
#endif
    hk_string tag="HK_VISIBLE";
    start_mastertag(s,tag);
    hk_string v="";
    switch (p_visibletype)
    {
        case   textlabel      :  v="TEXTLABEL";break;
        case button       :  v="BUTTON";break;
        case rowselector      :  v="SELECTOR";break;
        case boolean      :  v="BOOLEAN";break;
        case lineedit         :  v="LINEEDIT";break;
        case memo         :  v="MEMO";break;
        case combobox         :  v="COMBOBOX";break;
        case grid         :  v="GRID";break;
        case form         :  v="FORM";break;
        case report       :  v="REPORT";break;
        case reportsection    :  v="REPORTSECTION";break;
        case reportdata   :  v="REPORTDATA";break;
        default               : v="UNKNOWN";
    } ;
    set_tagvalue(s,"VISIBLETYPE",v);
    set_tagvalue(s,"LABEL",p_label);
    set_tagvalue(s,"X",(unsigned long ) p_x);
    set_tagvalue(s,"Y",(unsigned long ) p_y);
    set_tagvalue(s,"WIDTH",(unsigned long ) p_width);
    set_tagvalue(s,"HEIGHT",(unsigned long ) p_height);
    p_font.savedata(s);
    set_tagvalue(s,"VUPN",p_vupn);
    set_tagvalue(s,"COUNTS_AS",p_counts_as);

    hk_string coltag="FOREGROUNDCOLOUR";
    start_mastertag(s,coltag);
    p_foregroundcolour.savedata(s);
    end_mastertag(s,coltag);
    coltag="BACKGROUNDCOLOUR";
    start_mastertag(s,coltag);
    p_backgroundcolour.savedata(s);
    end_mastertag(s,coltag);

    hk_string align;
    switch (p_align)
    {
        case alignright:align="RIGHT";
        break;
        case aligncenter:align="CENTER";
        break;
        default    :align="LEFT";
    }
    set_tagvalue(s,"ALIGN",align);

    end_mastertag(s,tag);

}


void hk_visible::loaddata(const hk_string& definition)
{
#ifdef HK_DEBUG
    hkdebug("hk_visible::loaddata");
#endif
    hk_string v;
    get_tagvalue(definition,"VISIBLETYPE",v);
    get_tagvalue(definition,"LABEL",p_label);
    set_label(p_label);
    get_tagvalue(definition,"X", p_x);
    get_tagvalue(definition,"Y", p_y);
    get_tagvalue(definition,"WIDTH", p_width);
    get_tagvalue(definition,"HEIGHT", p_height);
    if (get_tagvalue(definition,"HK_FONT",v))
    {
     p_font.loaddata(v);
     set_font(p_font);
    }
    else
    {
      get_tagvalue(definition,"FONT",v);
    long int pfs;
    get_tagvalue(definition,"FONTSIZE",pfs);
    set_font(v,pfs);

    }

    get_tagvalue(definition,"COUNTS_AS",p_counts_as);
    long int p=-1;
    if (p_load_presentationnumber) if (get_tagvalue(definition,"VUPN",p))p_vupn=p;
    p_load_presentationnumber=true;
    set_size(p_x,p_y,p_width,p_height,false);
    hk_string align;
    get_tagvalue(definition,"ALIGN",align);
    if (align=="RIGHT") p_align=alignright;
    else if (align=="CENTER")p_align=aligncenter; else p_align=alignleft;
    set_alignment(p_align,false);

    hk_string col;
    if (get_tagvalue(definition,"FOREGROUNDCOLOUR",col))
    {
        p_foregroundcolour.loaddata(col);
        set_foregroundcolour(p_foregroundcolour,false);
    }
    if (get_tagvalue(definition,"BACKGROUNDCOLOUR",col))
    {
        p_backgroundcolour.loaddata(col);
        set_backgroundcolour(p_backgroundcolour,false);
    }

}


void hk_visible::has_changed(bool registerchange)
{
#ifdef HK_DEBUG
    hkdebug("hk_visible::has_changed(bool registerchange)");
#endif
    if ( (!registerchange)||(p_presentation==NULL)||p_neverregisterchange) return;

    p_presentation->set_has_changed();
}


void hk_visible::set_font(const hk_string& font,int size,bool registerchange)
{
    p_font.set_font(font,size);
    has_changed(registerchange);
    widget_specific_font_changed();
}


void hk_visible::set_font(const hk_font& font,bool registerchange)
{
    p_font=font;
    has_changed(registerchange);
    widget_specific_font_changed();
}


void    hk_visible::widget_specific_label_changed(void)
{

}


void hk_visible::set_neverregisterchange(bool n)
{
    p_neverregisterchange=n;
}


void hk_visible::set_datetimeformat(const hk_string& datetime,const hk_string& date, const hk_string&  time)
{
    p_datetimeformat=datetime;
    p_dateformat=date;
    p_timeformat=time;
}


hk_string hk_visible::datetimeformat(void)
{
    return p_datetimeformat;
}


hk_string hk_visible::dateformat(void)
{
    return p_dateformat;
}


hk_string hk_visible::timeformat(void)
{
    return p_timeformat;
}


unsigned long int hk_visible::counts_as(void)
{
    return p_counts_as;
}


void hk_visible::set_counts_as(unsigned long int c)
{
    p_counts_as=c;

}


hk_string hk_visible::replace(const hk_string& where)
{
#ifdef HK_DEBUG
    hkdebug("hk_visible::replace");
#endif
    hk_string p_buffer;
    p_buffer=replace_all("%COUNTS_AS%",where,longint2string(counts_as()));
    hk_datetime dt;
    dt.set_now();
    p_buffer=replace_all("%TIME%",p_buffer,dt.time_asstring());
    p_buffer=replace_all("%DATE%",p_buffer,dt.date_asstring());
    p_buffer=replace_all("%FGRED%",p_buffer,format_standard_number(p_foregroundcolour.red(),0,0));
    p_buffer=replace_all("%FGGREEN%",p_buffer,format_standard_number(p_foregroundcolour.green(),0,0));
    p_buffer=replace_all("%FGBLUE%",p_buffer,format_standard_number(p_foregroundcolour.blue(),0,0));
    p_buffer=replace_all("%BGRED%",p_buffer,format_standard_number(p_backgroundcolour.red(),0,0));
    p_buffer=replace_all("%BGGREEN%",p_buffer,format_standard_number(p_backgroundcolour.green(),0,0));
    p_buffer=replace_all("%BGBLUE%",p_buffer,format_standard_number(p_backgroundcolour.blue(),0,0));
    return p_buffer;

}


hk_presentation* hk_visible::presentation(void)
{
    return p_presentation;

}


void hk_visible::set_load_presentationnumber(bool l)
{
    p_load_presentationnumber=l;

}


bool hk_visible::load_presentationnumber(void)
{
    return p_load_presentationnumber;

}



void hk_visible::set_alignment(alignmenttype a,bool registerchange)
{
    p_align=a;
    widget_specific_alignment();
    has_changed(registerchange);
}


hk_visible::alignmenttype hk_visible::alignment(void)
{
    return p_align;
}


void hk_visible::widget_specific_alignment(void)
{

}


void hk_visible::set_defaulttextalignment(alignmenttype a)
{
    if (a==aligndefault)return;
    p_defaulttextalign=a;
}


hk_visible::alignmenttype hk_visible::defaulttextalignment(void)
{
    return p_defaulttextalign;

}


void hk_visible::set_defaultnumberalignment(alignmenttype a)
{
    if (a==aligndefault)return;
    p_defaultnumberalign=a;

}


hk_visible::alignmenttype hk_visible::defaultnumberalignment(void)
{

    return p_defaultnumberalign;
}


void hk_visible::set_open_maximized_windows(bool m)
{
    p_open_maximized_windows=m;
}


bool hk_visible::open_maximized_windows(void)
{
    return p_open_maximized_windows;
}


void   hk_visible::set_foregroundcolour(const hk_colour& colour,bool registerchange)
{

    set_foregroundcolour(colour,registerchange,false);
}


void   hk_visible::set_foregroundcolour(const hk_colour& colour,bool registerchange,bool force_setting)
{
#ifdef HK_DEBUG
    hkdebug("hk_visible::set_foregroundcolour");
#endif
    hk_colour oldcolour(p_foregroundcolour);
    p_tempforegroundcolour=colour;
    if (!(p_presentation && p_presentation->mode()==hk_presentation::view)||force_setting)
    {
        p_foregroundcolour=colour;
        has_changed(registerchange);
    }
    else
    {
        cerr <<"nur temporr foregroundcolour: "<<colour<<endl;
    }
    widget_specific_foregroundcolour_changed(oldcolour);

}


hk_colour hk_visible::foregroundcolour(void)
{
    if (p_presentation && p_presentation->mode()==hk_presentation::view) return p_tempforegroundcolour;
    return p_foregroundcolour;

}


void   hk_visible::set_backgroundcolour(const hk_colour& colour, bool registerchange)
{
    set_backgroundcolour(colour,registerchange,false);

}


void   hk_visible::set_backgroundcolour(const hk_colour& colour, bool registerchange,bool force_setting)
{
#ifdef HK_DEBUG
    hkdebug("hk_visible::set_backgroundcolour");
#endif
    hk_colour oldcolour(p_backgroundcolour);
    p_tempbackgroundcolour=colour;
    if (!(p_presentation && p_presentation->mode()==hk_presentation::view)||force_setting)
    {
        p_backgroundcolour=colour;
        has_changed(registerchange);
    }
    else
    {
        cerr <<"nur temporr backgroundcolour: "<<colour<<endl;
    }
    widget_specific_backgroundcolour_changed(oldcolour);
}


hk_colour hk_visible::backgroundcolour(void)
{
    if (p_presentation && p_presentation->mode()==hk_presentation::view) return p_tempbackgroundcolour;
    return p_backgroundcolour;
}


void    hk_visible::widget_specific_foregroundcolour_changed(const hk_colour&)
{

}


void    hk_visible::widget_specific_backgroundcolour_changed(const hk_colour& )
{

}


void hk_visible::lower_widget(bool registerchange)
{
    if (p_presentation) p_presentation->lower_widget(this);
    has_changed(registerchange);
    widget_specific_lower_widget();
}


void hk_visible::raise_widget(bool registerchange)
{
    if (p_presentation) p_presentation->raise_widget(this);
    has_changed(registerchange);
    widget_specific_raise_widget();

}


void hk_visible::widget_specific_raise_widget(void)
{

}


void hk_visible::widget_specific_lower_widget(void)
{

}


void hk_visible::sizetype_changed(void)
{
 if (!p_presentation) return;
 
 if (p_presentation->sizetype()==hk_presentation::relative)
  {
// this means previous sizetype was hk_presentation::absolute
// and changed now to hk_presentation::relative
   p_x=p_presentation->horizontal2relativ(p_x);
   p_y=p_presentation->vertical2relativ(p_y);
   p_width=p_presentation->horizontal2relativ(p_width);
   p_height=p_presentation->vertical2relativ(p_height);
  }
else
  {
// this means previous sizetype was hk_presentation::relative
// and changed now to hk_presentation::absolute
   p_x=p_presentation->relativ2horizontal(p_x);
   p_y=p_presentation->relativ2vertical(p_y);
   p_width=p_presentation->relativ2horizontal(p_width);
   p_height=p_presentation->relativ2vertical(p_height);

  }


widget_specific_sizetype_changed();
}


void        hk_visible::widget_specific_sizetype_changed(void)
{


}


